#ifndef COMPONENT_H_
#define COMPONENT_H_

#include <string>
#include <msp/datafile/objectloader.h>
#include <msp/fs/path.h>
#include "buildinfo.h"
#include "installmap.h"
#include "package.h"

class SourcePackage;

/**
Components specify things to be built.  Each component may build one binary (it
may also build none), as well as install a bunch of headers.  Components inherit
dependencies and build info from the package they belong to, and may also add
their own.
*/
class Component
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Component>
	{
	public:
		Loader(Component &);
	private:
		void build_info();
		void if_arch(const std::string &);
		void if_feature(const std::string &);
		void install_map();
		void overlay(const std::string &);
		void require(const std::string &);
		void source(const std::string &);
		void use(const std::string &);
	};

	enum Type
	{
		LIBRARY,
		PROGRAM,
		MODULE,
		DATAPACK,
		INSTALL,
		TARBALL
	};

	typedef std::list<Msp::FS::Path> SourceList;
	typedef std::list<std::string> OverlayList;
	typedef std::list<const Component *> UseList;

protected:
	SourcePackage &package;
	Type type;
	std::string name;
	SourceList sources;
	OverlayList overlays;
	bool install;
	BuildInfo build_info;
	Package::Requirements requires;
	UseList uses;
	bool deflt;
	InstallMap install_map;
	std::list<std::string> problems;

public:
	Component(SourcePackage &, Type, const std::string &);

	const SourcePackage &get_package() const { return package; }
	Type get_type() const { return type; }
	const std::string &get_name() const { return name; }

	/** Returns a list of sources for the component.  They may refer to
	directories or individual files. */
	const SourceList &get_sources() const { return sources; }

	const OverlayList &get_overlays() const { return overlays; }

protected:
	/** Returns a list of all source files for the component. */
	SourceList collect_source_files() const;

public:
	bool get_install() const { return install; }
	const InstallMap &get_install_map() const { return install_map; }
	const Package::Requirements &get_required_packages() const { return requires; }
	const UseList &get_used_components() const { return uses; }
	bool is_default() const { return deflt; }
	const std::list<std::string> &get_problems() const { return problems; }

	/** Prepares any required packages. */
	void prepare();

	/** Prepares the build information for building.  Pulls build info from the
	parent and dependency packages, and adds any component-specific flags. */
	void create_build_info();

	const BuildInfo &get_build_info() const { return build_info; }

	BuildInfo get_build_info_for_path(const Msp::FS::Path &) const;

	void create_targets() const;
};

#endif
