#ifndef EXTERNALTASK_H_
#define EXTERNALTASK_H_

#include <string>
#include <vector>
#include <msp/fs/path.h>
#include <msp/io/pipe.h>
#include "task.h"

/**
Runs an external command.  A zero exit status is translated to a SUCCESS status
for the task, and anything else is treated as an error.  Output can optionally
be captured.
*/
class ExternalTask: public Task
{
public:
	enum Destination
	{
		PASSTHROUGH,  //< Do not touch the stream
		CAPTURE,      //< Capture the stream
		IGNORE        //< Redirect the stream to oblivion
	};

	typedef std::vector<std::string> Arguments;

private:
	Arguments argv;
	Msp::FS::Path work_dir;
	int pid;
	int exit_code;
	Destination stdout_dest;
	Destination stderr_dest;
	Msp::IO::Pipe *capture_pipe;
	std::string output;

public:
	/** Creates an ExternalTask with an argument array and an optional working
	directory.  The first element of the argument array should be the command
	name.  If the working directory is not specified, no chdir is done. */
	ExternalTask(const Arguments &, const Msp::FS::Path & = Msp::FS::Path());

	virtual ~ExternalTask();

	virtual std::string get_command() const;
	virtual void start();
	virtual Status check();
	virtual Status wait();
private:
	Status do_wait(bool);

public:
	/// Sets destination for stdout.  Has no effect after the task is started.
	void set_stdout(Destination);

	/// Sets destination for stderr.  Has no effect after the task is started.
	void set_stderr(Destination);

	/** Returns captured output, if any.  This may be called while the task is
	still running, but it will always return all output. */
	const std::string &get_output() const { return output; }

	/** Executes a command and captures its output.  Stderr is ignored, but if
	the command exits with a nonzero status, an exception is thrown. */
	static std::string run_and_capture_output(const Arguments &, const Msp::FS::Path & = Msp::FS::Path());
};

#endif
